#!/usr/bin/env python

'''
Copyright 2013 Timur Rubeko

This file is part of Cliche.

Cliche is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Cliche is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Cliche.  If not, see <http://www.gnu.org/licenses/>.
'''

import os
import shutil
import string
import json
import logging

from .framework import shell


class TemplateError(Exception):
    '''Errors of this class represent issues in the templates, 
    rather than internal errors or issues in Cliche itself'''
    pass


class ExecutionEnvironmentError(Exception):
    '''Errors of this class represent errors in the working
    environment that prevent Cliche from proper functioning'''
    pass


class ApplicationBuilder(object):
    def __init__(self, args):
        self.args = args
        self.parameters = {}

    def build(self):
        self.mkdirs()
        self.clone()
        self.init_template()
        self.rename_paths()
        self.substitute_parameters_in_files()
        self.virtualenv()

    def mkdirs(self):
        if not os.path.isdir(self.project_path('.')):
            os.makedirs(self.project_path('.'))

    def clone(self):
        shell('git clone %s %s' % (self.args.template_repository_url, self.project_path('.')))
        shutil.rmtree(self.project_path('.git'))

    def init_template(self):
        '''Reads the `template.json` if it exists, parses defined substitution parameters
        and requests user (if not in --force-yes mode) values for those parameters'''
        if not os.path.isfile(self.project_path('template.json')): return
        template_config = json.load(open(self.project_path('template.json')))
        for parameter_name, default_value in template_config['parameters'].items():
            if not self.args.force_yes:
                user_input = raw_input('%s? [%s] ' % (parameter_name, default_value))
            parameter_value = not self.args.force_yes and user_input.strip() or default_value
            self.parameters[parameter_name] = parameter_value
        os.unlink(self.project_path('template.json')) # no need to keep it in generated app
        logging.debug('Parameters defined in the template: %s' % str(self.parameters))

    def virtualenv(self):
        if self.args.no_virtualenv: return

        shell('virtualenv %s' % self.project_path('venv'))
        if os.path.isfile(self.project_path('requirements.txt')):
            shell('%s/bin/pip install -r %s' % (self.project_path('venv'), self.project_path('requirements.txt')))

    def rename_paths(self):
        paths_to_rename = []

        for dir_path, dir_names, file_names in os.walk(self.project_path('.')):
            all_entry_names = []; all_entry_names.extend(dir_names); all_entry_names.extend(file_names)
            for entry_name in all_entry_names:
                substituted_entry_name = self.substitute_parameters(entry_name, location=entry_name)
                if substituted_entry_name != entry_name:
                    paths_to_rename.append(
                        (os.path.join(self.substitute_parameters(dir_path), entry_name),
                         os.path.join(self.substitute_parameters(dir_path), substituted_entry_name)))

        for source_path, dest_path in paths_to_rename:
            shutil.move(source_path, dest_path)

    def substitute_parameters_in_files(self):
        for dir_path, dir_names, file_names in os.walk(self.project_path('.')):
            for file_name in file_names:
                file_path = os.path.join(dir_path, file_name)
                with open(file_path, 'r') as f:
                    file_content = f.read()
                substituted_content = self.substitute_parameters(file_content, location=file_name)
                with open(file_path, 'w') as f:
                    f.write(substituted_content)

    def project_path(self, relative_path):
        '''For a relative file path inside an application to be created, returns actual path in the 
        file system. This is required in case current working directory is not the target new 
        application directory'''
        return os.path.normpath(os.path.join(self.args.target_directory_path, relative_path))

    def substitute_parameters(self, text, location=None):
        '''Substitutes application template parameter values into the given text. Requires `location`
        argument to log the erroneous file name in case of substitution errors'''
        try:
            return string.Template(text).substitute(self.parameters)
        except KeyError, err:
            raise TemplateError(
                "File '%s' contains a substitution pattern '%s', but it is not defined as a parameter for a template" % 
                (location, err.message))
