#!/usr/bin/env python

'''
Copyright 2013 Timur Rubeko

This file is part of Cliche.

Cliche is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Cliche is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Cliche.  If not, see <http://www.gnu.org/licenses/>.
'''

import os
import sys
import argparse
import subprocess
import logging

from .core import ApplicationBuilder, TemplateError, ExecutionEnvironmentError


LOG_FILE_PATH = os.path.expanduser('~/.cliche/cliche.log')


def verify_prerequisites():
    def verify_tool(error_messages, tool_name):
        exit_code = subprocess.call(tool_name + ' --version', 
            shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        return error_messages if exit_code == 0 else '%s%s is not available\n' % (error_messages, tool_name)
    return reduce(verify_tool, ('git', 'virtualenv'), '')


def verify_target_directory(target_directory_path):
    if os.path.isdir(target_directory_path) and os.listdir(target_directory_path):
        return 'Target directory `%s` is not empty. Cannot create an application in it' % target_directory_path
    else:
        return None


def create_parser():
    parser = argparse.ArgumentParser(
        description='Cliche creates a new application structure from a given template', 
        epilog='''Copyright 2013 Timur Rubeko. This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it under certain
conditions. For details see a copy of the GNU General Public License received,
or <http://www.gnu.org/licenses/>.''')
    parser.add_argument('-r', '--repo', dest='template_repository_url', required=True,
        help='an URL of the repository with an application template; e.g. https://guthub.com/user/repo')
    parser.add_argument('-d', '--directory', dest='target_directory_path', default='.',
        help='a path to the directory to create the application in; defaults to current directory')
    parser.add_argument('-y', '--force-yes', dest='force_yes', default=False, action='store_true',
        help='forces all default answers on any prompts, notably prompts for template parameter values')
    parser.add_argument('--no-virtualenv', dest='no_virtualenv', default=False, action='store_true',
        help='skips the creation of the virtualev (even if requirements.txt is present)')
    return parser


def configure_logging():
    cliche_dir_path = os.path.dirname(LOG_FILE_PATH)
    if not os.path.isdir(cliche_dir_path): os.makedirs(cliche_dir_path)
    logging.basicConfig(
        filename=LOG_FILE_PATH, 
        level=logging.DEBUG,
        format='[%(asctime)s] [%(levelname)5s] %(message)s',
        datefmt='%m/%d/%Y %H:%M:%S')


def run_workflow(argv):
    configure_logging()

    logging.info('Running cliche with following arguments: %s' % argv)

    parser = create_parser()
    args = parser.parse_args(argv[1:])

    prerequisites_missing = verify_prerequisites()
    if prerequisites_missing:
        raise ExecutionEnvironmentError(prerequisites_missing)

    target_directory_errors = verify_target_directory(args.target_directory_path)
    if target_directory_errors:
        raise ExecutionEnvironmentError(target_directory_errors)

    ApplicationBuilder(args).build()


def main(argv = sys.argv):
    try:
        run_workflow(argv)

    except TemplateError, err:
        print('''The template experienced an error while expanding it. Error message was:
%s
You can find the details in the Cliche log: '~/.cliche/cliche.log'. If you 
think this is an error in the template implementation, contact the template 
author about it. If you think, however, that this is a Cliche error - please, 
feel free to file an issue at <https://github.com/trubeko/cliche/issues>.''' % str(err))
        logging.exception('Template error')
        sys.exit(3)

    except ExecutionEnvironmentError, err:
        print('''Cliche cannot execute because of following error(s) in the environment:
%s
Please, fix the errors and try again. If you think, however, that this is a 
Cliche error - please, feel free to file an issue at 
<https://github.com/trubeko/cliche/issues>.''' % str(err))
        logging.exception('Environment error')
        sys.exit(2)

    except Exception, err:
        print('''Cliche experienced an internal error. Internal error message was:
%s
Please, find the error log in '~/.cliche/cliche.log' and feel free to file an
issue at <https://github.com/trubeko/cliche/issues>.''' % str(err))
        logging.exception('Internal error')
        sys.exit(1)


if __name__ == '__main__':
    main()
